@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -31,13 +31,21 @@ class BoolStrategy(SearchStrategy):
     def __repr__(self):
         return u'BoolStrategy()'
 
-    def calc_has_reusable_values(self):
+    def calc_has_reusable_values(self, recur):
         return True
 
     def do_draw(self, data):
         return d.boolean(data)
 
 
+def is_simple_data(value):
+    try:
+        hash(value)
+        return True
+    except TypeError:
+        return False
+
+
 class JustStrategy(SearchStrategy):
 
     """A strategy which simply returns a single fixed value with probability
@@ -50,9 +58,12 @@ class JustStrategy(SearchStrategy):
     def __repr__(self):
         return 'just(%r)' % (self.value,)
 
-    def calc_has_reusable_values(self):
+    def calc_has_reusable_values(self, recur):
         return True
 
+    def calc_is_cacheable(self, recur):
+        return is_simple_data(self.value)
+
     def do_draw(self, data):
         return self.value
 
@@ -86,8 +97,11 @@ class SampledFromStrategy(SearchStrategy):
         self.elements = d.check_sample(elements)
         assert self.elements
 
-    def calc_has_reusable_values(self):
+    def calc_has_reusable_values(self, recur):
         return True
 
+    def calc_is_cacheable(self, recur):
+        return is_simple_data(self.elements)
+
     def do_draw(self, data):
         return d.choice(data, self.elements)
